/****************************************************************************
**
** This file is part of a Qt Solutions component.
** 
** Copyright (c) 2009 Nokia Corporation and/or its subsidiary(-ies).
** 
** Contact:  Qt Software Information (qt-info@nokia.com)
** 
** Commercial Usage  
** Licensees holding valid Qt Commercial licenses may use this file in
** accordance with the Qt Solutions Commercial License Agreement provided
** with the Software or, alternatively, in accordance with the terms
** contained in a written agreement between you and Nokia.
** 
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
** 
** In addition, as a special exception, Nokia gives you certain
** additional rights. These rights are described in the Nokia Qt LGPL
** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
** package.
** 
** GNU General Public License Usage 
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
** 
** Please note Third Party Software included with Qt Solutions may impose
** additional restrictions and it is the user's responsibility to ensure
** that they have met the licensing requirements of the GPL, LGPL, or Qt
** Solutions Commercial license and the relevant license of the Third
** Party Software they are using.
** 
** If you are unsure which license is appropriate for your use, please
** contact the sales department at qt-sales@nokia.com.
** 
****************************************************************************/

#include "qactionstate.h"
#include "qactionstate_p.h"
#include "qstateaction.h"
#include "qstateaction_p.h"

QT_BEGIN_NAMESPACE

/*!
  \class QtActionState

  \brief The QtActionState class provides an action-based state.

  \ingroup statemachine

  QtActionState executes \l{QtStateAction}{state actions} when the state is
  entered and exited. QtActionState is part of \l{The State Machine Framework}.

  You can add actions to a state with the addEntryAction() and addExitAction()
  functions. The state executes the actions when the state is entered and
  exited, respectively.

  Built-in actions are provided for setting properties and invoking methods of
  QObjects. The setPropertyOnEntry() and setPropertyOnExit() functions are
  used for defining property assignments that should be performed when a state
  is entered and exited, respectively.

  \code
  QLabel label;
  QtStateMachine machine;
  QtState *s1 = new QtState();
  s1->setPropertyOnEntry(&label, "text"; "Entered state s1");
  machine.addState(s1);
  \endcode

  The invokeMethodOnEntry() and invokeMethodOnExit() functions are used for
  defining method invocations that should be performed when a state is entered
  and exited, respectively.

  \code
  QtState *s2 = new QtState();
  s2->invokeMethodOnEntry(&label, "showMaximized");
  machine.addState(s2);
  \endcode

  \sa QtStateAction
*/

/*!
   \enum QtActionState::RestorePolicy

   This enum specifies the restore policy type for a state. The restore policy takes effect when
   the machine enters a state which has entry actions of the type QtStateSetPropertyAction. If the 
   restore policy of the state is set to RestoreProperties, the state machine will save the 
   value of the property before the QtStateSetPropertyAction is executed. 

   Later, when the machine either enters a state which has its restore policy set to 
   DoNotRestoreProperties or when it enters a state which does not set a value for the given 
   property, the property will automatically be restored to its initial value. The state machine
   will only detect which properties are being set if they are being set using a
   QtStateSetPropertyAction object set as entry action on a state.

   Only one initial value will be saved for any given property. If a value for a property has 
   already been saved by the state machine, it will not be overwritten until the property has been
   successfully restored. Once the property has been restored, the state machine will clear the 
   initial value until it enters a new state which sets the property and which has RestoreProperties
   as its restore policy.

   \value GlobalRestorePolicy The restore policy for the state should be retrieved using 
          QtStateMachine::globalRestorePolicy()
   \value DoNotRestoreProperties The state machine should not save the initial values of properties 
          set in the state and restore them later.
   \value RestoreProperties The state machine should save the initial values of properties 
          set in the state and restore them later.


   \sa setRestorePolicy(), restorePolicy(), addEntryAction(), setPropertyOnEntry()
*/

QtActionStatePrivate::QtActionStatePrivate()
    : restorePolicy(QtActionState::GlobalRestorePolicy)
{
}

QtActionStatePrivate::~QtActionStatePrivate()
{
    qDeleteAll(entryActions);
    qDeleteAll(exitActions);
}

QtActionStatePrivate *QtActionStatePrivate::get(QtActionState *q)
{
    return q->d_func();
}

const QtActionStatePrivate *QtActionStatePrivate::get(const QtActionState *q)
{
    return q->d_func();
}

/*!
  Constructs a new action state with the given \a parent state.
*/
QtActionState::QtActionState(QtState *parent)
    : QtAbstractState(*new QtActionStatePrivate, parent)
{
}

/*!
  \internal
*/
QtActionState::QtActionState(QtActionStatePrivate &dd,
                           QtState *parent)
    : QtAbstractState(dd, parent)
{
}

/*!
  Destroys this action state.
*/
QtActionState::~QtActionState()
{
}

/*!
  Instructs this state to set the property with the given \a name of the given
  \a object to the given \a value when the state is entered. This function
  will create an object of QtStateSetPropertyAction and add it to the entry
  actions of the state.

  \sa setPropertyOnExit(), invokeMethodOnEntry(), addEntryAction()
*/
void QtActionState::setPropertyOnEntry(QObject *object, const char *name,
                                      const QVariant &value)
{
    addEntryAction(new QtStateSetPropertyAction(object, name, value));
}

/*!
  Instructs this state to set the property with the given \a name of the given
  \a object to the given \a value when the state is exited. This function will
  create an object of QtStateSetPropertyAction and add it to the exit actions
  of the state.

  \sa setPropertyOnEntry(), invokeMethodOnExit(), addExitAction()
*/
void QtActionState::setPropertyOnExit(QObject *object, const char *name,
                                     const QVariant &value)
{
    addExitAction(new QtStateSetPropertyAction(object, name, value));
}

/*!
  Instructs this state to invoke the given \a method of the given \a object
  with the given \a arguments when the state is entered. This function will
  create an object of QtStateInvokeMethodAction and add it to the entry actions
  of the state.

  \sa invokeMethodOnExit(), setPropertyOnEntry(), addEntryAction()
*/
void QtActionState::invokeMethodOnEntry(QObject *object, const char *method,
                                       const QList<QVariant> &arguments)
{
    addEntryAction(new QtStateInvokeMethodAction(object, method, arguments));
}

/*!
  Instructs this state to invoke the given \a method of the given \a object
  with the given \a arguments when the state is exited. This function will
  create an object of QtStateInvokeMethodAction and add it to the exit actions
  of the state.

  \sa invokeMethodOnEntry(), setPropertyOnExit(), addExitAction()
*/
void QtActionState::invokeMethodOnExit(QObject *object, const char *method,
                                      const QList<QVariant> &arguments)
{
    addExitAction(new QtStateInvokeMethodAction(object, method, arguments));
}

/*!
  Adds the given \a action to this state. The action will be executed when
  this state is entered. The state takes ownership of the action.

  \sa addExitAction()
*/
void QtActionState::addEntryAction(QtStateAction *action)
{
    Q_D(QtActionState);
    if (action)
        d->entryActions.append(action);
}

/*!
  Adds the given \a action to this state. The action will be executed when
  this state is exited. The state takes ownership of the action.

  \sa addEntryAction()
*/
void QtActionState::addExitAction(QtStateAction *action)
{
    Q_D(QtActionState);
    if (action)
        d->exitActions.append(action);
}

/*!
  Sets the restore policy of this state to \a restorePolicy. 
  
  The default restore policy is QtActionState::GlobalRestorePolicy.
*/
void QtActionState::setRestorePolicy(RestorePolicy restorePolicy)
{
    Q_D(QtActionState);
    d->restorePolicy = restorePolicy;
}

/*!
  Returns the restore policy for this state.
*/
QtActionState::RestorePolicy QtActionState::restorePolicy() const
{
    Q_D(const QtActionState);
    return d->restorePolicy;
}

/*!
  \reimp
*/
void QtActionState::onEntry()
{
    Q_D(QtActionState);
    for (int i = 0; i < d->entryActions.size(); ++i)
        QtStateActionPrivate::get(d->entryActions.at(i))->callExecute();
}

/*!
  \reimp
*/
void QtActionState::onExit()
{
    Q_D(QtActionState);
    for (int i = 0; i < d->exitActions.size(); ++i)
        QtStateActionPrivate::get(d->exitActions.at(i))->callExecute();
}

QT_END_NAMESPACE
